@@ -3,22 +3,22 @@
 #' The `makeClusterPSOCK()` function creates a cluster of \R workers
 #' for parallel processing.  These \R workers may be background \R sessions
 #' on the current machine, \R sessions on external machines (local or remote),
-#' or a mix of such. For external workers, the default is to use SSH to connect
-#' to those external machines.  This function works similarly to
+#' or a mix of such. For external workers, the default is to use SSH to
+#' connect to those external machines.  This function works similarly to
 #' \code{\link[parallel:makeCluster]{makePSOCKcluster}()} of the
-#' \pkg{parallel} package, but provides additional and more flexibility options
-#' for controlling the setup of the system calls that launch the background
-#' \R workers, and how to connect to external machines.
+#' \pkg{parallel} package, but provides additional and more flexibility
+#' options for controlling the setup of the system calls that launch the
+#' background \R workers, and how to connect to external machines.
 #'
-#' @param workers The hostnames of workers (as a character vector) or the number
-#' of localhost workers (as a positive integer).
+#' @param workers The hostnames of workers (as a character vector) or the
+#' number of localhost workers (as a positive integer).
 #' 
 #' @param makeNode A function that creates a `"SOCKnode"` or
 #' `"SOCK0node"` object, which represents a connection to a worker.
 #' 
 #' @param port The port number of the master used for communicating with all
-#' the workers (via socket connections).  If an integer vector of ports, then a
-#' random one among those is chosen.  If `"random"`, then a random port in
+#' the workers (via socket connections).  If an integer vector of ports, then
+#' a random one among those is chosen.  If `"random"`, then a random port in
 #' is chosen from `11000:11999`, or from the range specified by
 #' environment variable \env{R_PARALLELLY_RANDOM_PORTS}.
 #' If `"auto"` (default), then the default (single) port is taken from
@@ -41,11 +41,12 @@
 #' with `makeNode()` and the delay (in seconds) in-between attempts.
 #' If argument `port` specifies more than one port, e.g. `port = "random"`
 #' then a random port will be drawn and validated at most `tries` times.
-#' Arguments `tries` and `delay` are used only when `setup_strategy == "sequential`.
+#' Arguments `tries` and `delay` are used only when
+#' `setup_strategy == "sequential"`.
 #'
-#' @param validate If TRUE (default), after the nodes have been created, they are all
-#' validated that they work by inquiring about their session information,
-#' which is saved in attribute `session_info` of each node.
+#' @param validate If TRUE (default), after the nodes have been created,
+#' they are all validated that they work by inquiring about their session
+#' information, which is saved in attribute `session_info` of each node.
 #'
 #' @param verbose If TRUE, informative messages are outputted.
 #'
@@ -71,10 +72,15 @@ makeClusterPSOCK <- function(workers, makeNode = makeNodePSOCK, port = c("auto",
     if (length(workers) != 1L) {
       stopf("When numeric, argument 'workers' must be a single value: %s", length(workers))
     }
-    workers <- as.integer(workers)
+    
+    workers <- structure(as.integer(workers), class = class(workers))
     if (is.na(workers) || workers < 1L) {
       stopf("Number of 'workers' must be one or greater: %s", workers)
     }
+
+    ## Warn, or refuse to continue, if too many localhost workers are requests   
+    checkNumberOfLocalWorkers(workers)
+    
     workers <- rep(localhostHostname, times = workers)
   }
 
@@ -93,7 +99,11 @@ makeClusterPSOCK <- function(workers, makeNode = makeNodePSOCK, port = c("auto",
     free <- freeConnections()
     if (validate) free <- free - 1L
     if (length(workers) > free) {
-      stopf("Cannot create %d parallel PSOCK nodes. Each node needs one connection, but there are only %d connections left out of the maximum %d available on this R installation", length(workers), free, availableConnections())
+      msg <- sprintf("Cannot create %d parallel PSOCK nodes. Each node needs one connection, but there are only %d connections left out of the maximum %d available on this R installation", length(workers), free, availableConnections())
+      if (getRversion() >= "4.4.0") {
+        msg <- sprintf("%s. To increase this limit in R (>= 4.4.0), use command-line option '--max-connections=N' when launching R.", msg)
+      }
+      stopf(msg)
     }
   }
 
@@ -105,10 +115,14 @@ makeClusterPSOCK <- function(workers, makeNode = makeNodePSOCK, port = c("auto",
   if (length(port) == 0L) {
     stop("Argument 'port' must be of length one or more: 0")
   }
+  
+  port_org <- port
   port <- freePort(port)
   if (verbose) mdebugf("%sBase port: %d", verbose_prefix, port)
-
-
+  if (is.na(port)) {
+    stop("Argument 'port' specifies non-available port(s): ", paste(port_org, collapse = ", "))
+  }
+  
   n <- length(workers)
   nodeOptions <- vector("list", length = n)
   if (verbose) mdebugf("%sGetting setup options for %d cluster nodes ...", verbose_prefix, n)
